
#include <iostream>

#include <TApplication.h>
#include <TCanvas.h>
#include <TClass.h>
#include <TClassMenuItem.h>
#include <TFile.h>
#include <TGFileDialog.h>
#include <TKey.h>
#include <TSystem.h>

#include "MyGui.h"

ClassImp(MyMainFrame);
ClassImp(ROI);
ClassImp(ROICircle);
ClassImp(ROIEllipse);

const char *filetypes[] = { "ROOT files", "*.root", 0, 0 };

//----- The GUI ----------------------------------------------------------------------------

MyMainFrame::MyMainFrame(const TGWindow *p, UInt_t w, UInt_t h) : TGMainFrame(p,w,h) {
	Connect("CloseWindow()", "MyMainFrame", this, "CloseWindow()");
	SetWindowName("Right-click into image to access menu");

	fImg = (TASImage*)TImage::Open("rose512.jpg");
	fImg->SetConstRatio(kFALSE);

	// Create the embedded canvas
	fEmbCanvas = new TRootEmbeddedCanvas("Canvas1",this,512,512);
	AddFrame(fEmbCanvas, new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 0,0,0,0));
	fEmbCanvas->GetCanvas()->SetFillColor(1);
	fEmbCanvas->GetCanvas()->SetMargin(0,0,0,0);

	// Map main frame
	MapSubwindows();
	Resize(GetDefaultSize());
	MapWindow();

	// Add class menu entry to TASImage
	TClass *cl = TASImage::Class();
	cl->MakeCustomMenuList();
	TList *ml = cl->GetMenuList();
	ml->Clear();

	TClassMenuItem *item = new TClassMenuItem;
	item->SetType(TClassMenuItem::kPopupSeparator);

	ml->Add( new TClassMenuItem( TClassMenuItem::kPopupUserFunction, MyMainFrame::Class(), "Add Circular ROI"	, "AddROICircle"	, this ) );
	ml->Add( new TClassMenuItem( TClassMenuItem::kPopupUserFunction, MyMainFrame::Class(), "Add Elliptical ROI"	, "AddROIEllipse"	, this ) );
	ml->Add( item );
	ml->Add( new TClassMenuItem( TClassMenuItem::kPopupUserFunction, MyMainFrame::Class(), "Load ROIs"  		, "LoadROIs"  		, this ) );
	ml->Add( new TClassMenuItem( TClassMenuItem::kPopupUserFunction, MyMainFrame::Class(), "Save ROIs"  		, "SaveROIs"  		, this ) );
	ml->Add( item );
	ml->Add( new TClassMenuItem( TClassMenuItem::kPopupUserFunction, MyMainFrame::Class(), "Delete ROIs"		, "DeleteROIs"		, this ) );

	fImg->Draw();
}

void MyMainFrame::CloseWindow(){ gApplication->Terminate(); }

void MyMainFrame::AddROICircle(){
	fROIVector.push_back(new ROICircle);
	fROIVector.back()->Draw();
}

void MyMainFrame::AddROIEllipse(){
	fROIVector.push_back(new ROIEllipse);
	fROIVector.back()->Draw();
}

void MyMainFrame::DeleteROIs(){
	for(UShort_t iROI = 0; iROI < fROIVector.size(); iROI++) delete fROIVector[iROI];
	fROIVector.clear();
}

void MyMainFrame::LoadROIs(){
	TGFileInfo fi;
	fi.fFileTypes = filetypes;
	fi.fIniDir    = StrDup( gSystem->WorkingDirectory() );
	new TGFileDialog(gClient->GetRoot(), this, kFDOpen, &fi);
	if(!fi.fFilename) return;

	TFile fIn(fi.fFilename, "READ");
	if( !fIn.IsOpen() ){
		Error("MyMainFrame::LoadROIs()", ("Cannot open file " + std::string(fi.fFilename) + " !!!").c_str());
		return;
	}

	// ------ loop over ROI file keys ------
	ROI *roi = 0;
	TClass *classOfKey = 0;
	for(Int_t iKey = 0; iKey < fIn.GetListOfKeys()->GetSize(); iKey++ ){
		classOfKey = TClass::GetClass( ((TKey*)fIn.GetListOfKeys()->At(iKey))->GetClassName() );
		if(!classOfKey or !classOfKey->InheritsFrom(ROI::Class()) ) continue;

		roi = ROI::Create(classOfKey);
		if(roi){
			roi->Read( fIn.GetListOfKeys()->At(iKey)->GetName() );
			fROIVector.push_back(roi);
			roi->Draw();
		}
	}
	fIn.Close();
	fIn.DeleteAll();
}

void MyMainFrame::SaveROIs(){
	if(fROIVector.empty()) return;

	TGFileInfo fi;
	fi.fFileTypes = filetypes;
	fi.fIniDir    = StrDup( gSystem->WorkingDirectory() );
	new TGFileDialog(gClient->GetRoot(), this, kFDSave, &fi);
	if(!fi.fFilename) return;

	TString filename = fi.fFilename;
	if( !filename.EndsWith( ".root" ) ) filename.Append( ".root" );
	TFile fOut( filename, "RECREATE" );
	if( !fOut.IsWritable() ){
		Error("MyMainFrame::SaveROIs()", "No write access to file " + filename + " !!!");
		return;
	}

	for(UShort_t iROI = 0; iROI < fROIVector.size(); iROI++){
		TString roiname = "ROI";
		roiname += (iROI+1);
		fROIVector[iROI]->Write(roiname);
	}

	fOut.Close();
	fOut.DeleteAll();
}


//----- ROI (Region Of Interest) : Abstract base class -------------------------------------
ROI *ROI::Create(TClass *roiClass){
	if(!roiClass){
		std::cerr << "Error in <ROI::Create(TClass *roiClass)>: No ROI class specified! Returning NULL pointer." << std::endl;
		return 0;
	}
	if(roiClass == ROI::Class()){
		std::cerr << "Error in <ROI::Create(TClass *roiClass)>: Cannot create abstract base class object ROI! Returning NULL pointer." << std::endl;
		return 0;

	}
	if(!roiClass->InheritsFrom(ROI::Class())){
		std::cerr << "Error in <ROI::Create(TClass *roiClass)>: roiClass is not a child class of ROI! Returning NULL pointer." << std::endl;
		return 0;
	}

	TObject *obj = (TObject*)roiClass->New();
	ROI *roi = dynamic_cast<ROI*>(obj);

	if(roi) return roi;
	else delete obj;

	return 0;
}

//----- ROIEllipse -------------------------------------------------------------------------
ROIEllipse::ROIEllipse(Double_t x1NDC, Double_t y1NDC, Double_t r1NDC, Double_t r2NDC) : TEllipse(x1NDC,y1NDC,r1NDC,r2NDC) {
	SetLineColor(3);
	SetFillStyle(0);
	SetLineWidth(2);
}

ROIEllipse::ROIEllipse(const ROIEllipse &roi) : TEllipse(roi), ROI(roi) {}

ROIEllipse &ROIEllipse::operator=(const ROIEllipse &roiEllipse){
	roiEllipse.TEllipse::Copy(*this);
	return *this;
}

//----- ROICircle --------------------------------------------------------------------------
ROICircle &ROICircle::operator=(const ROICircle &roiCircle){
	roiCircle.TEllipse::Copy(*this);
	return *this;
}


void MyGui(){
	// pop up the GUI
	new MyMainFrame(gClient->GetRoot(), 600, 600);
}

int main(int argc, char **argv) {
	TApplication theApp("App",&argc,argv);
	MyGui();
	theApp.Run();
	return 0;
}
